#
# wrapper for net/imap
#

require 'net/imap'

#Net::IMAP::debug = true

class EmaiImap
  attr_accessor(:imap)
  def initialize(user, password, server, port, ssl)
    @handler      = nil
    @capabilities = nil
    @user, @password, @server, @port, @ssl = user, password, server, port, ssl
    login(user, password, server, port, ssl)
  end

  def login(user, password, server, port, ssl)
    if ssl then
      usessl = true; certs = nil; verify = false
      @imap = Net::IMAP.new(server, port, usessl, certs, verify)
      begin
        @imap.authenticate('CRAM-MD5', user, password)
      rescue
        @imap.authenticate('LOGIN', user, password)
      end
    else
      usessl = false; certs = nil; verify = false
      @imap = Net::IMAP.new(server, port, usessl, certs, verify)
      @imap.login(user, password)
    end
  end
  
  def set_handler
    @imap.remove_response_handler(@handler) if @handler
    @handler = Proc.new { |resp|
      if resp.kind_of?(Net::IMAP::UntaggedResponse)
        case resp.name
        when "EXISTS"
          $logger.info "Mailbox #{@mailbox} now has #{resp.data} messages"
        when "BYE"
          $logger.info "SERVER SAID BYE: #{resp.data}"
          $logger.info "RELOGIN"
          #logout()
          login(@user, @password, @server, @port, @ssl)
          @imap.select(@mailbox)
        when "STATUS"
          $logger.info "STATUS INFO: #{resp.data}"
        else
          $logger.info "INFO: #{resp.name}"
        end
      end
    }
    @imap.add_response_handler(@handler)
  end

  def select(mailbox)
    @mailbox = mailbox
    @imap.select(@mailbox)
    set_handler
  end

  def list(reflist, mailbox)
    @imap.list(reflist, mailbox)
  end

  def create(mailbox)
    @imap.create(mailbox)
  end

  def delete(mailbox)
    @imap.delete(mailbox)
  end

  def logout
    @imap.logout
  end

  def fetch(*args)
    @imap.fetch(*args)
  end

  def uid_fetch(*args)
    @imap.uid_fetch(*args)
  end

  def store(*args)
    @imap.store(*args)
  end

  def uid_store(*args)
    @imap.uid_store(*args)
  end

  def status(*args)
    @imap.status(*args)
  end

  def expunge()
    @imap.expunge()
  end

  def noop
    @imap.noop
  end

  def capability?(c)
    unless @capabilities
      @capabilities = @imap.capability()
    end
    return true if @capabilities.index(c)
    false
  end

  def thread()
    if capability?("THREAD=REFERENCES") then
      thread_by = "REFERENCES"
    elsif capability?("THREAD=ORDEREDSUBJECT") then
      thread_by = "ORDEREDSUBJECT"
    end
    begin
      threads = @imap.thread(thread_by, ["ALL"], "UTF-8")
    rescue
      begin
        threads = @imap.thread(thread_by, ["ALL"], "US-ASCII")
      rescue
        return nil # XXX fall back to self threading
      end
    end
  end
end

